(function(window){
	
	// Skip if already defined
	if(window.jef)
		return;
	
	// Function container
	var FunctionContainer = function() {
		
		var keyGen = 0;
		var functions = [];
	
		this.addFunction = function(key, fct){
			if( !jef.isString(key) ){
				fct = key;
				key = fct.key || (++keyGen);
			}
			fct.key = key;
			functions.push(fct);
			return fct;
		};
		
		this.removeFunction = function(key){
			for(var i=functions.length; i--;){
				if(functions[i].key == key){
					return functions.splice(i,1)[0];
				}
			}
		};
		
		this.getFunction = function(key){
			for(var i=functions.length, e; i--;){
				e = functions[i];
				if(e.key == key){
					return e;
				}
			}
		};
		
		this.callFunctions = function(context, args){
			// fix arguments						
			if( !jef.isArray(args) ){
				if( jef.isArray(context) ){
				    args = context;
					context = window;					
				} else {
					args = [];
					context = context || window;
				}
			} else {
				context = context || window;
			}
			for(var i=functions.length; i--; ){
				functions[i].apply(context, args);
			}
		};	
	};
	
	// Create Jef Object
	var jef = function(){ 
		
		var debug = true;		
		var debugUpdates = new FunctionContainer();
		
		var toString = Object.prototype.toString;
		var hasOwnProperty = Object.prototype.hasOwnProperty;
		
		var typeOf_ = function(value) {
		  var s = typeof value;
		  if (s == 'object') {
			if (value) {
			  if (value instanceof Array ||  
				  (!(value instanceof Object) &&
				   (toString.call(/** @type {Object} */ (value)) == '[object Array]') ||
				   typeof value.length == 'number' &&
				   typeof value.splice != 'undefined' &&
				   typeof value.propertyIsEnumerable != 'undefined' &&
				   !value.propertyIsEnumerable('splice')

				  )) {
				return 'array';
			  }
			  if (!(value instanceof Object) &&
				  (toString.call(/** @type {Object} */ (value)) == '[object Function]' ||
				  typeof value.call != 'undefined' &&
				  typeof value.propertyIsEnumerable != 'undefined' &&
				  !value.propertyIsEnumerable('call'))) {
				return 'function';
			  }

			} else {
			  return 'null';
			}
		  } else if (s == 'function' && typeof value.call == 'undefined') {
			return 'object';
		  }
		  return s;
		};
				
		var isSimpleObject_ = function(obj, objType){
			return objType == 'object' && 
				toString.call(obj) == '[object Object]' && 
				!obj.nodeType && 
				!obj.setInterval;
		};
		
		var extendOne_ = function(deep, dst, src, stack){
			var name, srcVal, srcValType, dstVal, dstValType, j, ref;
			if( deep ){
				stack.push(src);
			}
			for(name in src){
				srcVal = src[name];
				// skip if undefined or value = dst
				if ( srcVal === undefined || dst === srcVal){
					continue;
				}	
				// deep stuff
				if( deep && srcVal && ( // check for array or simple object
						(srcValType = typeOf_(srcVal)) == 'array' || 
						isSimpleObject_(srcVal, srcValType)
					)) {
					// check for reference, preventing for infinite loop
					for(ref=false, j=stack.length; j--;){
						if(stack[j] === srcVal){
							ref = true;
							break;
						}
					}
					// skip deep copy if ref found
					if( !ref ) {
						// get safe dst value
						dstVal = dst[name];
						dstVal = dstVal && (
							(dstValType = typeOf_(dstVal)) == 'array' || 
							isSimpleObject_(dstVal, dstValType)
						) ? dstVal : srcValType == 'array' ? [] : {};
						// go deeper						
						srcVal = extendOne_(deep, dstVal, srcVal, stack);						
					} 
				} 
				// assign value to dst object
				dst[name] = srcVal;
			}	
			if( deep ){
				stack.pop();
			}
			return dst;
		};
		
		return { 
			// debug
			setDebug : function(debug_){
				var log = jef.log, update = false;
				if( debug_ === undefined ){
					update = true;
				} else if( jef.isBoolean(debug_) ) {
					update = debug_ !== debug;
					if( update ){
						log.trace("[jef.setDebug] setting new debug value : " + debug);
					}
					debug = debug_; 
				} else {
					log.warn("[jef.setDebug] invalid debug value : " + debug);
				}
				if( update ){
					log.trace("[jef.setDebug] calling debug updates for debug value : " + debug);					
					debugUpdates.callFunctions(jef, [debug]);
				}
			},
			isDebug : function(){
				return debug;
			},
			addDebugUpdate : debugUpdates.addFunction,
			removeDebugUpdate : debugUpdates.removeFunction,
			getDebugUpdate : debugUpdates.getFunction,			
			// null & undefined	
			isDef : function(val) {
			  return val !== undefined;
			},
			isNull : function(val) {
			  return val === null;
			},
			isDefAndNotNull : function(val) {
			  return val != null;
			},
			// variable types
			/**
			 * This is a "fixed" version of the typeof operator.  It differs from the typeof
			 * operator in such a way that null returns 'null' and arrays return 'array'.
			 * @param {*} value The value to get the type of.
			 * @return {string} The name of the type.
			 */
			typeOf : typeOf_,
			isArray : function(val) {
			  return typeOf_(val) == 'array';
			},
			isArrayLike : function(val) {
			  var type = typeOf_(val);
			  return type == 'array' || type == 'object' && typeof val.length == 'number';
			},
			isDateLike : function(val) {
			  return jef.isObject(val) && typeof val.getFullYear == 'function';
			},
			isString : function(val) {
			  return typeof val == 'string';
			},
			isBoolean : function(val) {
			  return typeof val == 'boolean';
			},
			isNumber : function(val) {
			  return typeof val == 'number';
			},
			isFunction : function(val) {
			  return typeOf_(val) == 'function';
			},
			isObject : function(val) {
				var type = typeOf_(val);
				return type == 'object' || type == 'array' || type == 'function';
			},
			isSimpleObject : function( obj ){
				return isSimpleObject_(obj, typeOf_(obj));
			},
			isPlainObject: function( obj ) {
				// Must be an Object.
				// Because of IE, we also have to check the presence of the constructor property.
				// Make sure that DOM nodes and window objects don't pass through, as well
				if ( !obj || toString.call(obj) !== "[object Object]" || obj.nodeType || obj.setInterval ) {
					return false;
				}
				
				// Not own constructor property must be Object
				if ( obj.constructor
					&& !hasOwnProperty.call(obj, "constructor")
					&& !hasOwnProperty.call(obj.constructor.prototype, "isPrototypeOf") ) {
					return false;
				}
				
				// Own properties are enumerated firstly, so to speed up,
				// if last one is own, then all properties are own.
			
				var key;
				for ( key in obj ) {}
				
				return key === undefined || hasOwnProperty.call( obj, key );
			},
			// miscellaneous
			assert : function(condition, opt_message) {
				if (debug && !condition) {
					throw Error('Assertion failed' + (opt_message ? ': ' + opt_message : ''));
				}
			},
			noop : function(){},
			/** Extends a object with others. 
			* If first arguments is a boolean, it will determine the deep copy mode.
			* The next argument will be the destination object (created if not exist).
			* All next arguments are the sources objects.
			* @return the destination object, extended with all sources.
			*/
			extend :  function(){				
				var dst = arguments[0], i = 1, deep = false, stack, length = arguments.length, src;
				// Handle a deep copy situation
				if ( typeof dst === "boolean" ) {
					deep = dst;
					dst = arguments[1] || {};
					i = 2;
				} else if ( !dst ){
					dst = {};
				}
				// Init stack if deep copy
				if( deep ) {
					stack = [ dst ];
				}
				// Iterate over sources
				for(; i<length; i++){
					src = arguments[i];
					// skip null and undefined
					if( src == null ){
						continue;
					}	
					// extend dst with src
					extendOne_(deep, dst, src, stack);
				}
				return dst;			
			},
			delegate : function(){		
				var TMP = function(){};
				return function(obj){
					TMP.prototype = obj;
					var tmp = new TMP();
					TMP.prototype = null;					
					return tmp; 
				};
			}(),
			/** Navigate througt data
			* @param data Oject or array to be navigate througth
			* @param path String (coma separaated) or Array of attributes name
			* @param stack Array to be fill with found element in data. Optional
			* @return The found data, null otherwise
			*/ 
			navigate : function(data, path, stack){
				if(jef.isString(path)){
					path = path.split('.');
				}
				for(var i=0, len=atts.length; i<len; i++ ){
					data = data[atts[i]];
					if(stack){
						stack[i] = data;
					}
					if(data == null){
						return null;
					}
				}
				return data;
			}
		};
	}();
		
	//
	// Util
	//
	
	jef.util = function(jef){
		return {
			/** Valid a Rut */
			validRut : function(arg1, arg2, arg3){
				return new jef.util.Rut(arg1, arg2, arg3).valid;
			},
			/** Rut Class */
			Rut : function(jef){
			
				// Regex
				var rdv = /([k|K|0-9])$/;
			    var rrut = /^(\d{1,8})$/;
			    var rdot = /\./g;
			    var rsep = /(\d+)(\d{3})/;
				var rrutDv = /^(\d{1,8})-([k|K|0-9])$/;
			
				// Util Functions		
			    var processDv = function(rut){
			        var M=0,S=1;
			        for(;rut;rut=Math.floor(rut/10))
			            S=(S+rut%10*(9-M++%6))%11;
			        return String(S?S-1:'K');
			    };				    
				var putDot = function(arg1){
			    	while (rsep.test(arg1)) {
						arg1 = arg1.replace(rsep, '$1.$2');
					}
					return arg1;
			    };				    
			    var validRutDv = function(arg1, arg2, arg3){
			    	// Dot
			    	var dot = jef.isBoolean(arg2) ? arg2 : (jef.isBoolean(arg3) ? arg3 : false);
			    	if(dot) arg1 = arg1.replace(rdot, '');
			    	// arg1=rut, arg2=dv
			    	if(jef.isDef(arg2) && !jef.isBoolean(arg2)){
			    		return rrut.test(arg1) && rdv.test(arg2)
			    			&& (jef.string.equals(arg2, processDv(arg1), true));
			    	// arg1 = rut-dv
			    	} else {
			    		return rrutDv.test(arg1) && jef.string.equals(RegExp.$2, 
				        	processDv(RegExp.$1), true);
			    	}	    	
			    };
			
				// Class Constructor
				var cstr = function(arg1, arg2, arg3){
			    
					// Check first argument
					jef.assert(jef.isDef(arg1),'[jef.util.Rut] First argument must be defined.');
												
					// Class fields		
					this.rut;
				    this.dv;
				    this.dot;
				    this.rutDv;
				    this.valid;
				    
					// Parse and Valid
					if(jef.isDef(arg2)){			
						if(jef.isBoolean(arg2)){				
							this.dot = arg2;
							this.valid = validRutDv(arg1, this.dot);
							var split = (this.dot ? arg1.replace(rdot, '') : arg1).split('-');
							this.rut = split[0];
							this.dv = split[1];
						} else {
							this.rut = String(arg1);
							this.dv = String(arg2);
							this.dot = jef.isBoolean(arg3) ? arg3 : false;
							this.valid = validRutDv(this.rut, this.dv, this.dot);
						}
					} else {
						this.dot = false;
						this.valid =  validRutDv(arg1, this.dot);
						var split = arg1.split('-');
						this.rut = split[0];
						this.dv = split[1];
					}
					
					this.rutDv = (this.dot ? putDot(this.rut) : this.rut) + '-' + this.dv;
					this.toString = function(){
						return this.rutDv;
					};
					
				};
				
				// 'Static' functions 
				cstr.processDv = processDv;
				
				return cstr;
			
			}(jef),
			/**
			 * Add or override key|value data to a container.
			 * Object to be put to container may have both type :
			 * 
			 * 	1/ { 'key_name' : 'data_key', 'value_name' : 'data_value' }
			 * 	2/ { 'data_key' : 'data_value', 'data_key_2' : 'data_value_2', ... }
			 * 
			 * Where : 
			 * 	- key_name : see {@code options.key}
			 * 	- value_name : see {@code options.value}
			 * 	- data_key, data_key_2, ... : the data keys
			 * 	- data_value, data_value_2, ... : the data values
			 * 
			 * You also can put an Array of either type of object described before.
			 * 
			 * {@code options} are options for evaluating {@code patterns} :
			 * 	- key : the name of the attribute witch contains the data key (for type 1/). 
			 *  Default to 'key'
			 *  - value : the name of the attribute witch contains the data value (for type 1/). 
			 *  Default to 'value'
			 *  - prefix : will be used to filter data keys. It means that only data witch key starts with 
			 *  {@code opt_prefix} will be put to container. Default to ''
			 * 
			 * {@code options.key} is used to determine witch type of data (1/ or 2/) are being put 
			 * in container. If object has an attribute defined as {@code options.key}, type 1/ will
			 * be used.
			 *  
			 * @param {object} container The container to receive data 
			 * @param {object|Array.<object>} patterns Patterns to be add to container
			 * @param {object} options Options for evaluating patterns
			 */	
			putAllToContainer : function(container, data, options){
				if(!container || !data){
					jef.log.error('[jef.util.putAllToContainer] container and data must be defined.');
					return;
				}
				
				// Fix options
				options = options || {};
				var key_name = options.key || 'key';
				var value_name = options.value || 'value';
				var prefix = options.prefix || '';
					
				var name, obj;
				if(jef.isArray(data)){
					if(data.length == 0)
						return;
					if(jef.isDef(data[0][key_name])){
						for(var i=0; i<data.length; i++){
							obj = data[i];
							name = obj[key_name];
							if(name.indexOf(prefix) != 0)
								continue;
							container[name] = obj[value_name];
						}
					} else {
						for(var i=0; i<data.length; i++){
							obj = data[i];
							for(name in obj){
								if(name.indexOf(prefix) != 0)
									continue;
								container[name] = obj[name];
							}
						}
					}				
				} else {
					if(jef.isDef(data[key_name])){
						name = data[key_name];
						if(name.indexOf(prefix) == 0)
							container[name] = data[value_name];					
					} else {
						for(name in data){
							if(name.indexOf(prefix) != 0)
								continue;
							container[name] = data[name];
						}
					}
				}		
			},
			/** Function Container */
			FunctionContainer : FunctionContainer
		};
	}(jef);
	
	//
	// String
	//
	
	jef.string = function(jef){
	
		var trimRegex = /^(\s|\u00A0)+|(\s|\u00A0)+$/g;
		var trimLeftRegex = /^(\s|\u00A0)+|(\s|\u00A0)+$/;
		var trimRightRegex = /(\s|\u00A0)/;
		var lineBreakRegex = /\n/g;
		var toString = Object.prototype.toString;
		
		var valueToString_ = function(value, lineBreak, fctBody){
			if(value === undefined){
				return 'undefined';
			}
			if(value === null){
				return 'null';
			}	
			var type = jef.typeOf(value);
			if(type == 'string'){
				return '"' + value + '"';
			}
			if(type == 'number'){
				return value;
			}	
			if(type == 'function'){
				value = String(value);
				return fctBody ? value.replace(lineBreakRegex, lineBreak) :
					[value.substring(0, value.indexOf('{') + 1), 
					 value.substr(value.lastIndexOf('}'))].join('...');
			}
			return type == 'array' ? ('[' + value + ']') : value;
		};		
		
		var objToStr_ = function(object, options, stack, parts){
			// read options
			var own = options.own, deep = options.deep, fctBody = options.fctBody, skipNull = options.skipNull;
			// object type
			var type = jef.typeOf(object), isArray = type == 'array', 
				isFunction = type == 'function'; 
			// indentation
			var indTextOut = '', indTextIn = ' ', indent = isArray ? options.indentArray : options.indent;
			if( indent ){
				indTextOut = '\n' + new Array(stack.length).join('\t');
				indTextIn = indTextOut + '\t';
			}				
			// skip values			
			if( !isArray && toString.call(object) !== '[object Object]' ){
				parts.push(valueToString_(object, indTextOut, fctBody));
				if( !options.fctDeep || !isFunction ){ 
					return parts;
				}
			}						
			// open
			parts.push(isArray ? '[' : '{');
			// go through array / object
			var first = true, value, i, ref, stackPrefix = stack[stack.length - 1][0] + '.';
			for(attribute in object){
				// skip not own properties
				if( own && !object.hasOwnProperty(attribute) ) {
					continue;
				}
				// get value
				value = object[attribute];
				// skip null values
				if( skipNull && value == null ){
					continue;
				}
				// first flag
				if( !first ) {
					parts.push(',');
					parts.push(indTextIn);
				} else {
					first = false;
					// indentation
					if ( indent ){
						parts.push(indTextIn);
					}
				}
				// attribute name for object
				if( !isArray ){
					parts.push(attribute);
					parts.push(': ');
				}
				// deep stuff
				if( deep ){
					// check for deep reference, preventing for infinite loop
					for (ref = null, i = stack.length; i--; ) {
						if(stack[i][1] === value){
							ref = stack[i][0];
							break;
						}
					}
					if( ref ){	
						// ref text
						parts.push(ref);
					} else {
						// manage stack and go deeper
						stack.push([ stackPrefix + attribute, value]);
						objToStr_(value, options, stack, parts);
						stack.pop();
					}
				} else {
					// value to string
					parts.push(valueToString_(value, indTextOut, fctBody));
				}
			}	
			// last indent
			if( indent && !first ){
				parts.push(indTextOut);
			}
			// close
			if( isFunction && first ){
				parts.pop();
			} else {
				parts.push(isArray ? ']' : '}');
			}			
			return parts;
		};
		
		return {
			/**
			 * Test if two strings are equals.
			 * @param {string} txt_1 First text to be tested  
			 * @param {string} txt_2 Second text to be tested 
			 * @param {boolean} ignoreCase If case will be ignored 
			 * @return {boolean} True if strings are equals, false otherwise
			 */
			equals : function(txt_1, txt_2, ignoreCase){
				if( txt_1 == null || txt_2 == null)
					return false;
				if(ignoreCase == true){
					txt_1 = txt_1.toLowerCase();
					txt_2 = txt_2.toLowerCase();				
				}
				return txt_1 === txt_2;			 
			},	
			/**
			 * Test if a string is equals to one of other strings.
			 * @param {string} txt Text to be tested  
			 * @param {Array.<string>} compareTexts Others text to be compared with 
			 * @param {boolean} ignoreCase If case will be ignored 
			 * @return {boolean} True if string is contains, false otherwise
			 */		
			isIn : function(txt, compareTexts, ignoreCase){
				if(!jef.isDef(txt))
					return false;
				txt = ignoreCase == true ? txt.toLowerCase() : txt;
				ctxt = null;
				for(var i=0; i<compareTexts.length; i++){
					ctxt = compareTexts[i];
					if(ctxt == null)
						continue;
					ctxt = ignoreCase == true ? ctxt.toLowerCase() : ctxt;
					if(ctxt == txt)
						return true;
				}
				return false;
			},		
			/**
			 * Fix length of a given string. If it's length < {@code length}, then
			 * source will be filled with string {@code fillWith}, if > {@code length}
			 * source will be simply cut at {@code length}.
			 * @param {string} src The string source.
			 * @param {number} length The length of result string.
			 * @param {string} fillWith The string for filling source string.
			 * @return {string} A string with length {@code length}
			 */
			fixLength : function(src, length, fillWith){
				var dst = src + '';		
				fillWith = fillWith == null ? ' ' : String(fillWith);
				var left = length > 0;
				if (!left) {
					length = -length;
				}
				while(dst.length < length){
					dst = left ? (dst + fillWith) : (fillWith + dst);
				}
				if(dst.length > length){
					dst = left ? dst.substr(0, length) : dst.substr(dst.length - length, dst.length);
				}
				return dst;
			},
			/**
			 * Repeats a string n times.
			 * @param {string} string The string to repeat.
			 * @param {number} count The number of times to repeat.
			 * @return {string} A string containing {@code count} repetitions of
			 *     {@code string}.
			 */
			repeat : function(string, count) {
			  return new Array(count + 1).join(string);
			},
			/**
			 * Pads number to given length and optionally rounds it to a given precision.
			 * For example:
			 * <pre>padNumber(1.25, 2, 3) -> '01.250'
			 * padNumber(1.25, 2) -> '01.25'
			 * padNumber(1.25, 2, 1) -> '01.3'
			 * padNumber(1.25, 0) -> '1.25'</pre>
			 *
			 * @param {number} num The number to pad.
			 * @param {number} length The desired length.
			 * @param {number} opt_precision The desired precision.
			 * @return {string} {@code num} as a string with the given options.
			 */
			padNumber : function(num, length, opt_precision) {
			  var s = jef.isDef(opt_precision) ? num.toFixed(opt_precision) : String(num);
			  var index = s.indexOf('.');
			  if (index == -1) {
				index = s.length;
			  }
			  return new Array(Math.max(0, length - index) + 1).join('0') + s;
			},
			/**
			 * Trim a string.
			 * @param {string} text Text to be trimmed
			 * @return {string} The trimmed text
			 */
			trim : function(text){
				return text.replace(trimRegex, '');
			},
			trimLeft : function(text){
				return (text.length > 0 && text.charAt(0).match(trimRightRegex)) ? 
					text.replace(trimLeftRegex, '') : text;
			},
			trimRight : function(text){
				var li = text.length - 1;
				while(li >= 0 && text.charAt(li).match(trimRightRegex)){
					li--;
				}
				return text.substring(0,li+1);
			},
			/**
			 * Return a string representation of am object.
			 * @param {object} object The object
			 * @return {string} string The string representation
			 */
			objToStr : function(object, options) { 			
				// Fix options	
				options = options || {};
				options.indent = !!options.indent;
				options.own = !!options.own;
				options.deep = !!options.deep;
				options.fctBody = !!options.fctBody;	
				options.fctDeep = !!options.fctDeep;
				options.skipNull = !!options.skipNull;
				options.indentArray = options.indentArray == null ? options.indent : !!options.indentArray;
				// Init stack and parts
				var stack, parts = [];
				if( options.name ) {
					parts.push(options.name + ' = ');
					stack = [ ['#' + options.name, object] ];
				} else {
					stack = [ ['#this', object] ];
				}
				// Internal fct
				objToStr_(object, options, stack, parts); 
				return parts.join('');
			}		
		};
	}(jef);
			
	//
	// Arrays
	//
	
	jef.array = function(jef){		
		return {	
			copy : function(src, srcPos, dst, dstPos, length){
				srcPos = srcPos || 0;			
				dstPos = dstPos || 0;
				length = length || (src.length - srcPos);
				dst = dst || new Array(length + dstPos);
				for(var i=0; i < length; i++){
					dst[dstPos + i] = src[srcPos + i];
				}			
				return dst;
			},
			concat : function(/** target, sources... */) {
				var dst = arguments[0] || new Array();
				var length = arguments.length;
				var i = 1, src, j;			
				for ( ; i < length; i++ ) {
					src = arguments[i];
					if(src == null)
						continue;
					if(!jef.isArray(src)){
						dst.push(src);
						continue;
					}
					for(j=0; j<src.length; j++){
						dst.push(src[j]);
					}						
				}
				return dst;
			},
			contains : function(array, value) {
				for(var i=0; i<array.length; i++){
					if(array[i] === value)
						return true;
				}
				return false;
			},
			remove : function(array, value) {
				for(var i=0; i<array.length; i++){
					if(array[i] === value){
						array.splice(i,1);
						return true;
					}
				}
				return false;
			}
		};
	}(jef);
		
	//
	// i18n
	//
	
	jef.i18n = function(jef){
		
		// Locale, default to chile
		var locale = 'es_CL';
		var rlocale = /^([a-z]{2}|[a-z]{2}_[A-Z]{2})$/;
		
		/**
		 *  Get navigator's locale.
		 *  @param {string} def The default locale
		 *  @return {string} The navigator's locale or default one if not found.
		 */
		var getUserLocale = function(def){
			var locale;
			if ( navigator ) {
				if ( navigator.language ) {
					locale = navigator.language;
				}
				else if ( navigator.browserLanguage ) {
					locale = navigator.browserLanguage;
				}
				else if ( navigator.systemLanguage ) {
					locale = navigator.systemLanguage;
				}
				else if ( navigator.userLanguage ) {
					locale = navigator.userLanguage;
				}
			}
			return locale ? locale = locale.replace('-','_') : def;
		};
		/** Functions to be called when locale was set */
		var localeUpdates = new FunctionContainer();		
			
		return {			
			userlocale : getUserLocale(locale),
			getLocale : function(){
				return locale;
			},
			setLocale : function(locale_){
			    var log = jef.log, update = false;
				if( locale_ === undefined ){
					update = true;
				} else if( rlocale.test(locale_) ) {
					update = locale_ !== locale;
					if( update ){
						log.trace("[jef.i18n.setLocale] setting new locale : " + locale_);
					}
					locale = locale_; 
				} else {
					log.warn("[jef.i18n.setLocale] invalid locale : " + locale_);
				}
				if( update ){
					log.trace("[jef.i18n.setLocale] calling locale updates for locale : " + locale);					
					localeUpdates.callFunctions(jef.i18n, [locale]);
				}
			},
			addLocaleUpdate : localeUpdates.addFunction,
			removeLocaleUpdate : localeUpdates.removeFunction,
			getLocaleUpdate : localeUpdates.getFunction
		};
	}(jef);
			
	//
	// Format
	//
		
	jef.formatter = function(jef){		
		/**
		 * Formatter container
		 */
		var container = {
			/** Trim formatter */
			trim : function(pattern, options) {
				var doTrim = pattern == 1 ? jef.string.trimLeft : 
					(pattern == 2 ? jef.string.trimRight : jef.string.trim);
				this.format = function(value, options){					
					return doTrim(value);
				};		
				this.parse = function(text, options){
					return String(text);
				};		
				this.parseAtAllCost = function(text, options){
					return String(text);
				};
			},
			/** Upper case formatter */
			upper : function(pattern, options) {
				this.format = function(value, options){					
					return value.toUpperCase();
				};		
				this.parse = function(text, options){
					return String(text);
				};		
				this.parseAtAllCost = function(text, options){
					return String(text);
				};
			},
			/** Lower case formatter */
			lower : function(pattern, options) {
				this.format = function(value, options){					
					return value.toLowerCase();
				};		
				this.parse = function(text, options){
					return String(text);
				};		
				this.parseAtAllCost = function(text, options){
					return String(text);
				};
			},
			/** Substring by substr function */
			substr : function(pattern, options) {
				var indexes = pattern ? pattern.split(options && options.sep || '-') : [];
				this.format = function(value, options){					
					return value.substr.apply(value, indexes);
				};		
				this.parse = function(text, options){
					return String(text);
				};		
				this.parseAtAllCost = function(text, options){
					return String(text);
				};
			},
			/** Substring by substring function */
			substring : function(pattern, options) {
				var indexes = pattern ? pattern.split(options && options.sep || '-') : [];
				this.format = function(value, options){					
					return value.substring.apply(value, indexes);
				};		
				this.parse = function(text, options){
					return String(text);
				};		
				this.parseAtAllCost = function(text, options){
					return String(text);
				};
			},
			/** Replace */
			replace : function(){
				var rsep1 = /([^\\])(,)/g, rsep2 = /\\,/g;
				var sepr1 = '$1__coma__', sepr2 = '__coma__';			
				return function(pattern, options) {	
					var args = pattern ? pattern.replace(rsep1, sepr1).replace(rsep2,',').split(sepr2) : [], li, rpl;
					if( args.length > 0 ){
						rpl = args[0];
						if( rpl.indexOf('/') === 0 ){
							li = rpl.length;
							while( (li = rpl.lastIndexOf('/', li-1)) != -1 ){
								if(rpl.charAt(li-1) !== '\\')
									break;
							}	
							args[0]= li > 0 ? 
								args[0] = new RegExp(rpl.substring(1,li), rpl.substr(li+1)) : 
								args[0] = new RegExp(rpl.substr(1));
						}
					}
					if( options ){
						jef.extend(args, options);
					}
					this.format = function(value, options){	
						options = options ? jef.extend([], args, options) : args;						
						return value.replace.apply(value, options);
					};		
					this.parse = function(text, options){
						return String(text);
					};		
					this.parseAtAllCost = function(text, options){
						return String(text);
					};
				}
			}(),
			/** Object to string */
			object : function(pattern, options){
				var objToStr = jef.string.objToStr;
				var opt = pattern && options ? jef.extend(pattern, options) : (pattern || options);				
				this.format = function(value, options){					
					return objToStr(value, options != null ? jef.extend({}, opt, options) : opt);
				};		
				this.parse = function(text, options){
					return text;
				};		
				this.parseAtAllCost = function(text, options){
					return text;
				};
			},
			/** Rut */
			rut : function(pattern, options) {
				var Rut = jef.util.Rut;
				var dot = (!options || !jef.isBoolean(options.dot)) && pattern == 1 ? true : !!(options && options.dot);
				this.format = function(value, options){
					var dot_ = options && jef.isBoolean(options.dot) ? options.dot : dot;
					var rut = value instanceof Rut ? value : new Rut(value, dot_);
					return rut.valid ? rut.toString() : '';
				};		
				this.parse = function(text, options){
					var dot_ = options && jef.isBoolean(options.dot) ? options.dot : dot;
					var rut = new Rut(text, dot_);
					return rut.valid ? rut : null;
				};		
				this.parseAtAllCost = function(text, options){
					var dot_ = options && jef.isBoolean(options.dot) ? options.dot : dot;
					var rut = new Rut(text, true);			
					return rut.valid ? new Rut(rut.rut, rut.dv, dot_) : null;
				};
				this.invalidMessage = 'invalid.format.rut';
			}
		};
		
		return {
			/** Put new formatter to container */
			put : function(key, constructor){
				container[key] = constructor;
			},
			/** Get formatter from container */
			get : function(key) {
				return container[key];
			}, 
			/** Delete formatter from container */
			remove : function(key){
				return delete container[key];
			}			
		};
	}(jef);
	
	jef.format = function(jef, util, formatter){
		/**
		 * Container of patterned format.
		 */
		var container = {
			// String format
			'trim' : 0,
			'trim.left' : 1,
			'trim.right' : 2,
			'upper' : null,
			'lower' : null,
			'substr' : '0',
			'substr.first' : '0-1',
			'substring' : '0',
			'replace' : null,
			'object' : null,
			'object.own' : { own: true },
			'object.deep' : { deep: true },
			'object.full' : { indent: true, deep: true, fctDeep: true },
			'object.full.vect' : { indent: true, deep: true, fctDeep: true, indentArray: false },
			'object.full.fct' : { indent: true, deep: true, fctDeep: true, fctBody: true },
			'object.full.own' : { indent: true, deep: true, fctDeep: true, own: true },
			// Number
			'number' : '0',
			'number.dec' : '0.00',
			'number.sep' : '#,##0',
			'number.sep.dec' : '#,##0.00',
			'number.money' : '#,##0',
			'number.money.dec' : '#,##0.00',
			'number.currency' : '¤ #,##0',
			'number.currency.dec' : '¤ #,##0.00',
			// Date
			'date' : 'dd/MM/yyyy',
			'date.datetime' : 'dd/MM/yyyy HH:mm:ss',
			// Time
			'time' :'HH:mm:ss',
			// Rut
			'rut' : 0,
			'rut.dot' : 1	
		};

		var getFormatter_ = function(formatKey, options){
			var pattern = container[formatKey];
			if(pattern === undefined){
				jef.log.error('[jef.format.getFormatter] Pattern not found for key : ' + formatKey);
				return null;
			}
			var formatterKey = formatKey.split('.')[0];
			var Formatter = formatter.get(formatterKey);
			if(!Formatter){
				jef.log.error('[jef.format.getFormatter] Formatter not found for key : ' + formatterKey);
				return null;
			}				
			var fmtr = new Formatter(pattern, options);
			// Fix parseAtAllCost function
			if(!fmtr.parseAtAllCost){
				fmtr.parseAtAllCost = fmtr.parse;
			}
			return fmtr;
		};
		
		var fct = function(value, formatKey, options){
			var formatter = getFormatter_(formatKey, options);
			return formatter ? formatter.format(value, options) : null;
		};
		
		return jef.extend(fct, {
			/** All patterns key */
			//ALL : 'all',
			/**
			 * Add or override one patterns format to container.
			 */
			put : function(key, pattern){
				container[key] = pattern;
			},
			/**
			 * Add or override one or many patterns formats to container.
			 * Object to be put to container may have both type :
			 * 
			 * 	1/ { 'key_name' : 'pattern_key', 'value_name' : 'pattern_value' }
			 * 	2/ { 'pattern_key' : 'pattern_value', 'pattern_key_2' : 'pattern_value_2', ... }
			 * 
			 * Where : 
			 * 	- key_name : see {@code options.key}
			 * 	- value_name : see {@code options.value}
			 * 	- pattern_key, pattern_key_2, ... : the pattern keys
			 * 	- pattern_value, pattern_value_2, ... : the pattern values
			 * 
			 * You also can put an Array of either type of object described before.
			 * 
			 * {@code options} are options for evaluating {@code patterns} :
			 * 	- key : the name of the attribute witch contains the pattern key (for pattern type 1/). 
			 *  Default to 'key'
			 *  - value : the name of the attribute witch contains the pattern value (for pattern type 1/). 
			 *  Default to 'value'
			 *  - prefix : will be used to filter pattern keys. It means that only pattern witch key starts with 
			 *  {@code opt_prefix} will be put to container. Default to ''
			 * 
			 * {@code options.key} is used to determine witch type of pattern (1/ or 2/) are being put 
			 * in container. If object has an attribute defined as {@code options.key}, type 1/ will
			 * be used.
			 *  
			 * @param {object|Array.<object>} patterns Patterns to be add to container
			 * @param {object} options Options for evaluating patterns
			 */
			putAll : function(patterns, options) {						
				util.putAllToContainer(container, patterns, options);
			},	
			/** Get formatter from container */
			getFormatter : getFormatter_,
			/**
			 * General function for getting format pattern from in jef.format#container. 
			 * If {@code formatKey} is not given, process will find for {@code prefix} key in container.
			 * If {@code formatKey} is a not a string, it will be directly return because the format implementation
			 * should be able to deal with; if is string it will look in container with key {@code prefix + '.' + formatKey} 
			 * and then with {@code formatKey}.
			 * If {@code formatKey} is not found in container, it will be return.
			 * @param {string|anything|null} formatKey Container pattern key, self-define pattern string or 
			 * constant pattern number.
			 * @prefix {string} Prefix used to look in container, used as default key if {@code formatKey} is not given.
			 * @return {string|anything} The string pattern or constant pattern number
			 */
			getPattern : function(formatKey, prefix){
				if(formatKey == null) {
					// Use default pattern
					return container[prefix];
				} 
				if (typeof formatKey == 'string'){
					// Use one of container pattern
					var containerPattern;
					if(prefix != null){
						containerPattern = container[prefix + '.' + formatKey];
						if(containerPattern){
							return containerPattern;
						}
					}
					containerPattern = container[formatKey];
					if(containerPattern){
						return containerPattern;
					}
				} 
				return formatKey;
			},
			/** Get all patterns in container, filtered by prefix */
			getPatterns : function(prefix){
				var patterns = [];
				for(key in container){						
					if(key.indexOf(prefix) === 0)
						patterns.push(container[key]);
				}
				return patterns;
			},
			/** Format text */
			format : fct,
			/** Parse value */
			parse : function(text, formatKey, options){
				var formatter = getFormatter_(formatKey, options);
				return formatter ? formatter.parse(text, options) : null;
			},
			/** Parse at all cost a value */
			parseAtAllCost : function(text, formatKey, options){
				var formatter = getFormatter_(formatKey, options);
				return formatter ? formatter.parseAtAllCost(text, options) : null;
			}
		});		
	
	}(jef, jef.util, jef.formatter);
	
	//
	// Log
	//
		
	jef.log = function(jef, window){		
		
		//
		// loggers
		//
		var fixLength = jef.string.fixLength;
		var defaultFormat = function(message, level){
			var date = new Date();	
			var h = fixLength(date.getHours(),-2,'0');
			var m = fixLength(date.getMinutes(),-2,'0');
			var s = fixLength(date.getSeconds(),-2,'0');
			var u = fixLength(date.getMilliseconds(),-3,'0');
			return h + ':' + m + ':' + s + ',' + u + ' [' + level + '] ' + message;
		};
		
		var keyGen = 0;
		var container = {
			noop : { key : 'noop', log : jef.noop, format : jef.noop },
			alert : {
				key : 'alert',
				log : function(message, level){ if(level > 3) window.alert(message); },
				format : defaultFormat
			},
			console : (function(){
				var _cons, _logs;
				var initLogs = function(){
					_cons = window.console;
					_logs = _cons && [_cons.trace || _cons.log, 
					                  _cons.debug || _cons.log, 
					                  _cons.info || _cons.log,
					                  _cons.warn || _cons.log,
					                  _cons.error || _cons.log]; 
					return !!_logs;
				};				
				return {
					key : 'console',
					log : ( initLogs() ? 
						function(message, level)
						{
							(_logs[level-1] || _cons.log).apply(_cons, message);
						} : 
						function(message, level)
						{
							if(!_logs && !initLogs()) {
								return;
							};
							(_logs[level-1] || _cons.log).apply(_cons, message);
						}),
					format : function(message, level) {
						return [defaultFormat("", level), message];
					}
				};
			})()
		};		
		var currentLogger = window.console ? container.console : container.noop;
						
		// Text for level
		var TXT = ['TRACE','DEBUG','INFO','WARN','ERROR'];
		// Current level
		var currentLevel = 2;
		// Previous level (usefull for disable/enable)
		var previousLevel = 2;
		
		// setLevel
		var setLevel_  = function(level){
			level = isNaN(level) ? currentLevel : ( (level = +level) < 1 || level > 6 ? currentLevel : level);
			// secure that currentLevel != previousLevel
			// this  prevent from having previousLevel = currentLevel = 6
			// witch will break the expected effect of the enable() function
			if(level !== currentLevel){
				previousLevel = currentLevel;
				currentLevel = level;
				logEnabled = jef.isDebug() && currentLevel !== 6;
			}
		};		
				
		// jef.log as function
		var logEnabled = true;
		var fct = function(message, level)
		{		
			if( logEnabled && (level = (+level || 2)) >= currentLevel && level < 6) {			
				currentLogger.log(currentLogger.format(message, TXT[level-1]), level);
			}
		};
		
		// Add debug update
		jef.addDebugUpdate('i18n', function(debug){			
			logEnabled = debug && currentLevel !== 6;
		});
		
		return jef.extend(fct, {			
			// Levels
			DISABLED : 6,
			ERROR : 5,
			WARN : 4,
			INFO : 3,
			DEBUG : 2,
			TRACE : 1,
			// Current level		
			setLevel : setLevel_,
			getLevel : function(){
				return currentLevel;
			},			
			// Disable / enabled
			disable : function(){
				setLevel_(6);
			},
			enable : function(){
				if(currentLevel === 6){
					setLevel_(previousLevel);
				}
			},
			// Level implementation
			error : function(message){ fct(message,5); },
			warn : function(message){ fct(message,4); },
			info : function(message){ fct(message,3); },	
			debug : function(message){ fct(message,2); },
			trace : function(message){ fct(message,1); },			
			// Level test
			isErrorEnabled : function(){ return logEnabled && currentLevel <= 5; },	
			isWarnEnabled : function(){ return logEnabled && currentLevel <= 4; },	
			isInfoEnabled : function(){ return logEnabled && currentLevel <= 3; },	
			isDebugEnabled : function(){ return logEnabled && currentLevel <= 2; },
			isTraceEnabled : function(){ return logEnabled && currentLevel <= 1; },					
			isLevelEnabled : function(level) {
				return logEnabled && (level = (+level || 2)) >= currentLevel && level < 6;
			},
			/** Add logger to container */
			addLogger : function(key, logger){
				if(!jef.isString(key)){
					logger = key;
					key = logger.key || 'logger_' + (++keyGen);					
				}
				if( jef.isFunction(logger) ){
					logger = {
						log : logger
					};
				} else if( !jef.isFunction(logger.log) ){
					return null;
				}
				if( !jef.isFunction(logger.format) ){
					logger.format = defaultFormat;
				}
				if( !jef.isString(key) ){
					key = 'logger_' + (++keyGen);
				}
				logger.key = key;
				container[key] = logger;				
				return logger;
			},
			/** Set as current and possibly add it to container */
			setLogger : function(key, logger){
				currentLogger = ( !logger && jef.isString(key) ? container[key] : 
					jef.logger.addLogger(key, logger) ) || container.noop;				
			},
			/** Remove logger from container */
			removeLogger : function(key){
				return delete container[key];
			},
			/** Get logger from container, o current logger if key isn't defined */
			getLogger : function(key){
				return key ? container[key] : currentLogger;
			}	
		});
	}(jef, window);
		
	//
	// Message
	//	
	
	jef.msg = function(jef, format, formatter, util){
		/**
		 * Container of key/value messages
		 */
		var container = {
			'invalid.fieldvalue' : 'El valor ingresado para el campo {fieldName} es inválido',
			'incorrect.fieldvalue' : 'El valor ingresado para el campo {fieldName} es incorrecto',
			'invalid.format.date' : 'La fecha es inválida',
			'invalid.format.time' : 'La hora es inválida',
			'invalid.format.number' : 'El número es inválido',
			'invalid.format.rut': 'El Rut es inválido'
		};
				
		var format_ = function(text, data, commentEnable){
			// Fix arguments
			var hasData;
			if( jef.isBoolean(data) ){
				commentEnable = data;
				hasData = false;
			} else {			
				hasData = !!data;
				commentEnable = jef.isBoolean(commentEnable) ? commentEnable : false;
			}
			// Skip if nothing to parse
			if(!hasData && !commentEnable)
				return text;			
				
			var index = 0, result = '', commentIndex, dataIndex;
			while(index < text.length){
							
				commentIndex = commentEnable ? text.indexOf('\'', index) : -1;
				dataIndex = hasData ? text.indexOf('{', index) : -1;
				
				// Neither data and comment are found, flush to end
				if(dataIndex == -1 && commentIndex == -1){
					result += text.substring(index, text.length);
					return result;
				}
				
				// If data found first
				if(dataIndex != -1 && (commentIndex == -1 || dataIndex < commentIndex)){
					// flush to dataIndex
					result += text.substring(index, dataIndex);
					// go next
					index = dataIndex + 1;
					if(index >= text.length)
						return result + '{';
					// Search end of data
					dataIndex = text.indexOf('}', index);
					// No end found, data start will be flush
					if(dataIndex == -1){
						result += '{';
						continue;
					}
					// Search data value
					var textData = text.substring(index, dataIndex);
					result += formatData_(textData, data);
					index = dataIndex + 1;
					continue;					
				}
				
				// If comment found
				if(commentIndex != -1){
					// flush to commentIndex
					result += text.substring(index, commentIndex);					
					// go next
					index = commentIndex + 1;
					if(index >= text.length)
						return result;
					// Search end of comment
					commentIndex = text.indexOf('\'', index);
					// If no end found, flush to end
					if(commentIndex == -1){
						result += text.substring(index, text.length);
						return result;
					}
					// If next one, flush only one comment char
					if(commentIndex == index){
						result += '\'';
						index ++;
						continue;
					}
					// Else, flush all comment content
					result += text.substring(index, commentIndex);
					index = commentIndex + 1;
					continue;
				}
			}			
			return result;		
		};
		
		var getData_ = function(data, attPath){
			var atts = attPath.split('.');
			for(var i=0, len=atts.length; i<len; i++ ){
				data = data[atts[i]];
				if(data == null){
					return null;
				}
			}
			return data;
		};
		
		var formatData_ = function(textData, data){
			// Split textData
			var split = textData.split(','), length = split.length;
			// Get value
			var value = getData_(data, split[0]);
			if(value == null)
				return '{' + textData + '}';
			// No format defined
			if(length == 1)
				return String(value);
			// Fix length > 3
			if(length > 3){
				for(var i=3; i<length; i++){
					split[2] += ',' + split[i];
				}
			}
			// Get format type and pattern
			var formatType = split[1]; 
			var pattern = length === 2 ? null : split[2];
			// Get formatter	
			var Formatter = formatter.get(formatType);
			if(Formatter == null)
				return String(value);
			// Get pattern from container and instance formatter
			pattern = format.getPattern(+pattern || pattern, formatType);
			// Format value and return 
			var formatValue = new Formatter(pattern).format(value);
			if(formatValue != null)
				return formatValue;
			return String(value);
		};
		
		var rnkey = /(\s|\u00A0|<)/;
		
		var fct = function(key, data, commentEnable){
			if( !rnkey.test(key) ) {
				key = container[key] || key;
			}			
			return format_(key, data, commentEnable);
		};
		
		return jef.extend(fct, {		
			get : function(key, data, commentEnable){
				var text = container[key];
				return text ? format_(text, data, commentEnable) : key;		
			},
			put : function(key, message){
				container[key] = message;
			},
			putAll : function(messages, options){
				util.putAllToContainer(container, messages, options);
			},
			has : function(key){
				return !!container[key];
			},
			format : format_
		});
		
	}(jef, jef.format, jef.formatter, jef.util);
		
	//
	// Abstract Format
	//
	
	jef.format.AbstractFormat = function(impl){
		
		var format = jef.format;
		var array = jef.array;
	
		var staticKeyGen_ = 0;
		var staticFormatters_ = [];
		
		var key = impl.key;
		var failResult = impl.failResult !== undefined ? impl.failResult : null;
		var isNotValid = impl.isNotValid || function(value) { 
			return value === failResult; 
		};
		var invalidMessage = impl.invalidMessage || ('invalid.format.' + key);
		
		var newFormatter_ = impl.newFormatter;
		var getFormatter_ = impl.getFormatter || function(pattern, options){
			pattern = format.getPattern(pattern, key);
			return newFormatter_(pattern, options);
		};
			
		this.getFormatter = getFormatter_;
		
		this.addStaticFormatter = function(arg1, arg2){
			var formatter;
			if( jef.isSimpleObject(arg1) && jef.isFunction(arg1.parse) && jef.isFunction(arg1.format) ){
				formatter = arg1;
			} else {
				formatter = newFormatter_(arg1, arg2);
				formatter.key = arg2 && arg2.key;
			}				
			formatter.key = formatter.key || (++staticKeyGen_);
			staticFormatters_.push(formatter);
			return formatter;
		};
		this.removeStaticFormatter = function(key){
			for(var i=staticFormatters_.length; i--;){
				if(staticFormatters_[i].key == key){
					return staticFormatters_.splice(i,1)[0];
				}
			}
		};
		this.getStaticFormatter = function(key){
			for(var i=staticFormatters_.length, e; i--;){
				e = staticFormatters_[i];
				if(e.key == key){
					return e;
				}
			}
		};
	
		var parseAtAllCost_ = function(text, options){
			options = options || {};
			var result, i = 0, len = staticFormatters_.length;
			for(; i<len; i++){						
				result = staticFormatters_[i].parse(text, options);
				if(!isNotValid(result))
					return result;
			}	
			var patterns = format.getPatterns(key);
			i = 0, len = patterns.length;
			for(; i<len; i++){	
				result = newFormatter_(patterns[i], options).parse(text, options);
				if(!isNotValid(result))
					return result;
			}
			return failResult;
		};
		
		this.format = function(value, pattern, options){
			return getFormatter_(pattern, options).format(value, options);
		},
		
		this.parse = function(text, pattern, options){	
			return getFormatter_(pattern, options).parse(text, options);
		},
		
		this.parseAtAllCost = parseAtAllCost_;
		
		/**
		 * Add Formatter
		 */		
		jef.formatter.put(key, function(pattern, options) {			
			var opt = options || {};
			var f = newFormatter_(pattern, opt);
			var pos = opt.pos ? array.copy(opt.pos) : undefined;
			this.parse = function(text, options){
				if( !options ){
					opt.pos = pos ? array.copy(pos) : undefined;
					options = opt;
				}
				var res = f.parse(text, options);
				return isNotValid(res) ? null : res;
			};		
			this.format = function(value, options){
				return f.format(value, options || opt);
			};		
			this.parseAtAllCost = function(text, options){
				if( !options ){
					opt.pos = pos ? array.copy(pos) : undefined;
					options = opt;
				}				
				var res = parseAtAllCost_(text, options);
				return isNotValid(res) ? null : res;
			};
			this.invalidMessage = invalidMessage;		
		});
		
	};
	
	//
	// Number
	//
	
	jef.number = function(jef){
		
		// Shortcut
		var getPattern_ = jef.format.getPattern;
		
		// Empty number format provider, need to be set before
		var NumberFormat = function(pattern, options){
			jef.log.error("[jef.number#NumberFormat] No NumberFormatProvider is set !");
			this.parse = function(text, options){
				jef.log.error("[jef.number#NumberFormat.parse] No Number Format Provider is set !");
				return null;
			};
			this.format = function(number) {
				jef.log.error("[jef.number#NumberFormat.format] No Number Format Provider is set !");
				return '';
			};
		};
				
		return jef.extend(new jef.format.AbstractFormat(
			{
				key : 'number',
				failResult : NaN,
				isNotValid : isNaN,			
				newFormatter : function(pattern, options){
					return new NumberFormat(pattern, options);
				},
				getFormatter : function(pattern, options){
					return new NumberFormat(getPattern_(pattern, 'number'), options);
				}
			}), {
			/** Get Number Format provider */
			getProvider : function(){
				return NumberFormat;
			},
			/** Set Number Format provider */
			setProvider : function(NumberFormatProvider){
				NumberFormat = NumberFormatProvider;
			},
			round : function(value, nDec){
				if( !nDec ) {
					return Math.round(value);
				} else {
					var e = Math.pow(10, nDec);
					return Math.round(value * e) / e;
				}
			},
			floor : function(value, nDec){
				if( !nDec ) {
					return Math.floor(value);
				} else {
					var e = Math.pow(10, nDec);
					return Math.floor(value * e) / e;
				}
			},
			ceil : function(value, nDec){
				if( !nDec ) {
					return Math.ceil(value);
				} else {
					var e = Math.pow(10, nDec);
					return Math.ceil(value * e) / e;
				}
			}
		});
	
	}(jef);
	
	//
	// Date / Time
	//
	
	var dateTime = function(jef){
	
		// Shortcut
		var getPattern_ = jef.format.getPattern;
	
		// Empty date time format provider
		var DateTimeFormat = function(pattern, options){
			jef.log.error("[jef.date#DateTimeFormat] No DateTime Format Provider is set !");
			this.parse = function(text, options){
				jef.log.error("[jef.date#DateTimeFormat.parse] No DateTime Format Provider is set !");
				return null;
			};
			this.format = function(number) {
				jef.log.error("[jef.date#DateTimeFormat.format] No DateTime Format Provider is set !");
				return '';
			};
		};
			
		var buildComparableDate_ = function(d, ignoreTime, ignoreMilliseconds){
			if( typeof d == 'number' ){
				d = new Date(d);
			}
			if( ignoreTime === true ){
				return new Date(d.getFullYear(),d.getMonth(),d.getDate(), 0,0,0,0);
			}
			if( ignoreMilliseconds === true ){
				return new Date(d.getFullYear(),d.getMonth(),d.getDate(),
						d.getHours(),d.getMinutes(),d.getSeconds(), 0);
			}
			return d;
		};
		
		var buildComparableTime_ = function(d, ignoreMilliseconds){
			if( typeof d == 'number' ){
				d = new Date(d);
			}
			if(ignoreMilliseconds === true){
				return new Date(2000,0,1, d.getHours(),d.getMinutes(),d.getSeconds() ,0);
			}
			return new Date(2000,0,1, d.getHours(),d.getMinutes(),d.getSeconds(),d.getMilliseconds());
		};
			
		return [
		// Date
		jef.extend(new jef.format.AbstractFormat(
			{
				key : 'date',
				newFormatter : function(pattern, options){
					return new DateTimeFormat(pattern, options);
				},
				getFormatter : function(pattern, options){
					return new DateTimeFormat(getPattern_(pattern, 'date'), options);
				}
			}), {
			/** Get DateTime Format provider */
			getProvider : function(){
				return DateTimeFormat;
			},
			/** Set DateTime Format provider */
			setProvider : function(DateTimeFormatProvider){
				DateTimeFormat = DateTimeFormatProvider;
			},			
			/** Clone a Date */
			clone : function(date){
				return new Date(date.getTime());
			},
			/** Determine if two dates are equals */
			equals : function(date1, date2, ignoreTime, ignoreMilliseconds){
				date1 = buildComparableDate_(date1, ignoreTime, ignoreMilliseconds).getTime();
				date2 = buildComparableDate_(date2, ignoreTime, ignoreMilliseconds).getTime();
				return date1 === date2;
			}
		}),
		// Time
		jef.extend(new jef.format.AbstractFormat(
			{
				key : 'time',
				newFormatter : function(pattern, options){
					return new DateTimeFormat(pattern, options);
				},
				getFormatter : function(pattern, options){
					return new DateTimeFormat(getPattern_(pattern, 'time'), options);
				}
			}), {
			// No provider defined for jef.time, internally use the same as jef.date
			/** Determine if two times are equals */
			equals : function(time1, time2, ignoreMilliseconds){
				time1 = buildComparableTime_(time1, ignoreMilliseconds).getTime();
				time2 = buildComparableTime_(time2, ignoreMilliseconds).getTime();
				return time1 === time2;
			}
		})];
	
	}(jef);
	
	jef.date = dateTime[0];
	jef.time = dateTime[1];
		
	// Provide shortcuts from jef namespace
	jef.trim = jef.string.trim;
	jef.objToStr = jef.string.objToStr;
	jef.tpl = jef.msg.format;
	jef.parse = jef.format.parse;
	
	// Log and Format	
	jef.logf = function(){
		var log_ = jef.log;
		var format_ = jef.format;
		var isLevelEnabled_ = jef.log.isLevelEnabled;
		return function(value, formatKey, options, level){
			// Fix arguments
			if ( !level && typeof options == 'number' ){
				level = options;
				options = null;
			}
			// Test, Format and Log
			if(isLevelEnabled_(level)){
				return log_(format_(value, formatKey, options), level);
			}
		};
	}();
	
	// Log and Message	
	jef.logm = function(){
		var log_ = jef.log;
		var msg_ = jef.msg;
		var isLevelEnabled_ = jef.log.isLevelEnabled;
		return function(key, data, commentEnable, level){
			// Fix arguments
			if ( level == null ){
				if ( typeof commentEnable == 'number' ) {
					level = commentEnable;
					if ( typeof data == 'boolean' ){
						commentEnable = data;
						data = null;
					} else {
						commentEnable = null;
					}
				} else if ( commentEnable == null && data != null){
					var dtype = typeof data;
					if ( dtype == 'number' ){
						level = data;
						data = null;
					} else if ( dtype == 'boolean' ) {
						commentEnable = data;
						data = null;
					}
				}
			}
			// Test, Format and Log
			if(isLevelEnabled_(level)){
				return log_(msg_(key, data, commentEnable), level);
			}
		};
	}();
		
	// Expose Jef to the global object
	window.jef = jef;
	
})(window);